@@ -24,13 +24,18 @@
 
 #include <config.h>
 #include <string.h>
+#include <time.h>
 
 #include <glib-object.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 #include <panel-applet.h>
 #include <panel-applet-gconf.h>
 #include <gconf/gconf-client.h>
-#include <libgnomeui/libgnomeui.h>
+#include <libgnomeui/gnome-file-entry.h>
+#include <libgnomeui/gnome-help.h>
+#include <libgnomeui/gnome-pixmap-entry.h>
+#include <libgnomeui/gnome-uidefs.h>
 #include <libart_lgpl/libart.h>
 #include <glade/glade-xml.h>
 
@@ -79,8 +84,12 @@ typedef struct {
 	GtkWidget         *fortune_dialog;
 	GtkWidget         *fortune_view;
 	GtkWidget         *fortune_label;
+	GtkWidget         *fortune_cmd_label;
 	GtkTextBuffer	  *fortune_buffer;
 
+	unsigned int       source_id;
+	GIOChannel        *io_channel;
+
 	gboolean           april_fools;
 
 	guint              listeners [N_FISH_PREFS];
@@ -95,6 +104,7 @@ static gboolean load_fish_image          (FishApplet *fish);
 static void     update_pixmap            (FishApplet *fish);
 static void     something_fishy_going_on (FishApplet *fish,
 					  const char *message);
+static void     display_fortune_dialog   (FishApplet *fish);
 static void     set_tooltip              (FishApplet *fish);
 
 static GType fish_applet_get_type (void);
@@ -331,10 +341,6 @@ display_preferences_dialog (BonoboUIComponent *uic,
 	gtk_dialog_set_default_response (
 		GTK_DIALOG (fish->preferences_dialog), GTK_RESPONSE_OK);
 
-	gnome_window_icon_set_from_file (
-		GTK_WINDOW (fish->preferences_dialog),
-		GNOME_ICONDIR "/gnome-fish.png");
-
 	fish->name_entry = glade_xml_get_widget (xml, "name_entry");
 	gtk_entry_set_text (GTK_ENTRY (fish->name_entry), fish->name);
 
@@ -348,6 +354,8 @@ display_preferences_dialog (BonoboUIComponent *uic,
 			   "name" /* key */);
 
 	fish->pixmap_entry = glade_xml_get_widget (xml, "image_entry");
+	gnome_pixmap_entry_set_pixmap_subdir (GNOME_PIXMAP_ENTRY (fish->pixmap_entry),
+					    FISH_ICONDIR);
 	fish->image_entry = gnome_file_entry_gtk_entry (
 				GNOME_FILE_ENTRY (fish->pixmap_entry));
 	gtk_entry_set_text (GTK_ENTRY (fish->image_entry), fish->image);
@@ -468,10 +476,10 @@ display_about_dialog (BonoboUIComponent *uic,
 		"Sun GNOME Documentation Team <gdocteam@sun.com>",
           	NULL
 	};
+	/* Translator credits */
+	const char *translator_credits = _("translator-credits");
+
 	char        *authors [3];
-	GdkPixbuf   *pixbuf;
-	GError      *error = NULL;
-	char        *file;
 	char        *descr;
 
 	if (fish->about_dialog) {
@@ -485,40 +493,27 @@ display_about_dialog (BonoboUIComponent *uic,
 	authors [1] = _("(with minor help from George)");
 	authors [2] = NULL;
 
-	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
-					  "gnome-fish.png", FALSE, NULL);
-	pixbuf = gdk_pixbuf_new_from_file (file, &error);
-	g_free (file);
-
-	if (error) {
-		g_warning (G_STRLOC ": cannot open %s: %s", file, error->message);
-		g_error_free (error);
-	}
-
 	descr = g_strdup_printf (about_format, fish->name);
 		
-	fish->about_dialog =
-		gnome_about_new (_("Fish"),
-				 "3.4.7.4ac19",
-				 "Copyright \xc2\xa9 1998-2002 Free Software Foundation, Inc.",
-				 descr,
-				 (const char **) authors,
-				 documenters,
-				 NULL,
-				 pixbuf);
+	fish->about_dialog = gtk_about_dialog_new ();
+	g_object_set (fish->about_dialog,
+		      "name", _("Fish"),
+		      "version", "3.4.7.4ac19",
+		      "copyright", "Copyright \xc2\xa9 1998-2002 Free Software Foundation, Inc.",
+		      "comments", descr,
+		      "authors", (const char **) authors,
+		      "documenters", documenters,
+		      "translator_credits", strcmp (translator_credits, "translator-credits") != 0 ? translator_credits : NULL,
+		      "logo_icon_name", "gnome-fish",
+		      NULL);
 
 	g_free (descr);
 	g_free (authors [0]);
 
-	if (pixbuf)
-		g_object_unref (pixbuf);
-
 	gtk_window_set_wmclass (
 		GTK_WINDOW (fish->about_dialog), "fish", "Fish");
 	gtk_window_set_screen (GTK_WINDOW (fish->about_dialog),
 			       gtk_widget_get_screen (GTK_WIDGET (fish)));
-	gnome_window_icon_set_from_file (GTK_WINDOW (fish->about_dialog),
-					 GNOME_ICONDIR "/gnome-fish.png");
 
 	g_signal_connect (fish->about_dialog, "destroy",
 			  G_CALLBACK (gtk_widget_destroyed),
@@ -571,33 +566,50 @@ something_fishy_going_on (FishApplet *fish,
 	gtk_widget_show (dialog);
 }
 
-static char *
-locate_fortune_command (FishApplet *fish)
+static gboolean
+locate_fortune_command (FishApplet   *fish,
+			int          *argcp,
+			char       ***argvp)
 {
-	char *retval = NULL;
+	char *prog = NULL;
 
-	if (!fish->command) {
-		something_fishy_going_on (
-			fish, _("Unable to get the name of the command to execute"));
-		return NULL;
+	if (fish->command
+	    && g_shell_parse_argv (fish->command, argcp, argvp, NULL)) {
+		prog = g_find_program_in_path ((*argvp)[0]);
+		if (prog) {
+			g_free (prog);
+			return TRUE;
+		}
+
+		g_strfreev (*argvp);
 	}
 
-	if (g_path_is_absolute (fish->command))
-		retval = g_strdup (fish->command);
-	else
-		retval = g_find_program_in_path (fish->command);
+	prog = g_find_program_in_path ("fortune");
+	if (prog) {
+		g_free (prog);
+		if (g_shell_parse_argv ("fortune", argcp, argvp, NULL))
+			return FALSE;
+	}
 
-	if (!retval)
-		retval = g_find_program_in_path ("fortune");
+	if (g_file_test ("/usr/games/fortune", G_FILE_TEST_IS_EXECUTABLE)
+	    && g_shell_parse_argv ("/usr/games/fortune", argcp, argvp, NULL))
+		return FALSE;
 
-	if (!retval && g_file_test ("/usr/games/fortune", G_FILE_TEST_EXISTS))
-		retval = g_strdup ("/usr/games/fortune");
+	something_fishy_going_on (fish,
+				  _("Unable to locate the command to execute"));
+	*argvp = NULL;
+	return FALSE;
+}
 
-	if (!retval)
-		something_fishy_going_on (
-			fish, _("Unable to locate the command to execute"));
-
-	return retval;
+#define FISH_RESPONSE_SPEAK 1
+static inline void
+fish_close_channel (FishApplet *fish)
+{
+	if (fish->io_channel) {
+		g_io_channel_shutdown (fish->io_channel, TRUE, NULL);
+		g_io_channel_unref (fish->io_channel);
+	}
+	fish->io_channel = NULL;
 }
 
 static void
@@ -605,7 +617,17 @@ handle_fortune_response (GtkWidget  *widget,
 			 int         id,
 			 FishApplet *fish)
 {
-	gtk_widget_hide (fish->fortune_dialog);
+	if (id == FISH_RESPONSE_SPEAK)
+		display_fortune_dialog (fish);
+	else {
+		/* if there is still a pipe, close it: if we hide the widget,
+		 * the * output can't be seen */
+		if (fish->source_id)
+			g_source_remove (fish->source_id);
+		fish->source_id = 0;
+		fish_close_channel (fish);
+		gtk_widget_hide (fish->fortune_dialog);
+	}
 }
 
 static void
@@ -642,8 +664,12 @@ insert_fortune_text (FishApplet *fish,
 
 	gtk_text_buffer_get_iter_at_offset (fish->fortune_buffer, &iter, -1);
 
-	gtk_text_buffer_insert (fish->fortune_buffer, &iter,
-				text, -1);
+	gtk_text_buffer_insert_with_tags_by_name (fish->fortune_buffer, &iter,
+						  text, -1, "monospace_tag",
+						  NULL);
+
+	while (gtk_events_pending ())
+	  gtk_main_iteration ();
 }
 
 static void
@@ -655,21 +681,93 @@ clear_fortune_text (FishApplet *fish)
 	gtk_text_buffer_get_iter_at_offset (fish->fortune_buffer, &end, -1);
 
 	gtk_text_buffer_delete (fish->fortune_buffer, &begin, &end);
+	gtk_text_buffer_remove_tag_by_name (fish->fortune_buffer,
+					    "monospace_tag", &begin, &end);
 
 	/* insert an empty line */
 	insert_fortune_text (fish, "\n");
 }
 
+static gboolean
+fish_read_output (GIOChannel   *source,
+		  GIOCondition  condition,
+		  gpointer      data)
+{
+	char        output[4096];
+	char       *utf8_output;
+	gsize       bytes_read;
+	GError     *error = NULL;
+	GIOStatus   status;
+	FishApplet *fish;
+
+	fish = (FishApplet *) data;
+
+	if (!(condition & G_IO_IN)) {
+		fish->source_id = 0;
+		fish_close_channel (fish);
+		return FALSE;
+	}
+
+	status = g_io_channel_read_chars (source, output, 4096, &bytes_read,
+					  &error);
+
+	if (error) {
+		char *message;
+
+		message = g_strdup_printf (_("Unable to read output from command\n\nDetails: %s"),
+					   error->message);
+		something_fishy_going_on (fish, message);
+		g_free (message);
+		g_error_free (error);
+		fish->source_id = 0;
+		fish_close_channel (fish);
+		return FALSE;
+	}
+
+	if (status == G_IO_STATUS_AGAIN)
+		return TRUE;
+
+	if (bytes_read > 0) {
+		/* The output is not guarantied to be in UTF-8 format, most
+		 * likely it's just in ASCII-7 or in the user locale
+		 */
+		if (!g_utf8_validate (output, -1, NULL))
+			utf8_output = g_locale_to_utf8 (output, bytes_read,
+							NULL, NULL, NULL);
+		else
+			utf8_output = g_strndup (output, bytes_read);
+
+		if (utf8_output)
+			insert_fortune_text (fish, utf8_output);
+
+		g_free (utf8_output);
+	}
+
+	if (status == G_IO_STATUS_EOF) {
+		fish->source_id = 0;
+		fish_close_channel (fish);
+	}
+	return (status != G_IO_STATUS_EOF);
+}
+
 static void 
 display_fortune_dialog (FishApplet *fish)
 {
-	GError *error = NULL;
-	char   *fortune_command;
-	char   *output = NULL;
-	char   *utf8_output;
+	GError      *error = NULL;
+	gboolean     user_command;
+	int          output;
+	const char  *charset;
+	int          argc;
+	char       **argv;
 
-	fortune_command = locate_fortune_command (fish);
-	if (!fortune_command)
+	/* if there is still a pipe, close it */
+	if (fish->source_id)
+		g_source_remove (fish->source_id);
+	fish->source_id = 0;
+	fish_close_channel (fish);
+
+	user_command = locate_fortune_command (fish, &argc, &argv);
+	if (!argv)
 		return;
 
 	if (!fish->fortune_dialog) {
@@ -681,9 +779,13 @@ display_fortune_dialog (FishApplet *fish)
 		fish->fortune_dialog = 
 			gtk_dialog_new_with_buttons (
 				"", NULL, 0,
+				_("_Speak again"), FISH_RESPONSE_SPEAK,
 				GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 				NULL);
 
+		gtk_dialog_set_has_separator (
+			GTK_DIALOG (fish->fortune_dialog), FALSE);
+
 		gtk_dialog_set_default_response (
 			GTK_DIALOG (fish->fortune_dialog), GTK_RESPONSE_CLOSE);
 
@@ -693,16 +795,12 @@ display_fortune_dialog (FishApplet *fish)
 				  G_CALLBACK (handle_fortune_response), fish);
 
 		gtk_window_set_wmclass (GTK_WINDOW (fish->fortune_dialog), "fish", "Fish");
-		gnome_window_icon_set_from_file (GTK_WINDOW (fish->fortune_dialog),
-						 GNOME_ICONDIR"/gnome-fish.png");
 
 		screen = gtk_widget_get_screen (GTK_WIDGET (fish));
 
 		screen_width  = gdk_screen_get_width (screen);
 		screen_height = gdk_screen_get_height (screen);
 
-		gtk_window_set_screen (GTK_WINDOW (fish->fortune_dialog), screen);
-
 		gtk_window_set_default_size (GTK_WINDOW (fish->fortune_dialog),
 					     MIN (600, screen_width  * 0.9),
 					     MIN (350, screen_height * 0.9));
@@ -715,6 +813,10 @@ display_fortune_dialog (FishApplet *fish)
 		fish->fortune_buffer =
 			gtk_text_view_get_buffer (GTK_TEXT_VIEW (fish->fortune_view));
 
+		gtk_text_buffer_create_tag (GTK_TEXT_BUFFER (fish->fortune_buffer),
+					    "monospace_tag", "family",
+					    "Monospace", NULL);
+
 		scrolled = gtk_scrolled_window_new (NULL, NULL);
 		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
 						GTK_POLICY_AUTOMATIC,
@@ -725,6 +827,11 @@ display_fortune_dialog (FishApplet *fish)
 		gtk_container_add (GTK_CONTAINER (scrolled), fish->fortune_view);
 
 		fish->fortune_label = gtk_label_new ("");
+		gtk_label_set_ellipsize (GTK_LABEL (fish->fortune_label),
+					 PANGO_ELLIPSIZE_MIDDLE);
+		fish->fortune_cmd_label = gtk_label_new ("");
+		gtk_misc_set_alignment (GTK_MISC (fish->fortune_cmd_label),
+					0, 0.5);
 
 		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (fish->fortune_dialog)->vbox), 
 				    fish->fortune_label,
@@ -734,49 +841,82 @@ display_fortune_dialog (FishApplet *fish)
 				    scrolled,
 				    TRUE, TRUE, GNOME_PAD);
 
+		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (fish->fortune_dialog)->vbox), 
+				    fish->fortune_cmd_label,
+				    FALSE, FALSE, GNOME_PAD);
+
 		update_fortune_dialog (fish);
 
-		gtk_widget_show_all (fish->fortune_dialog);
+		/* We don't show_all for the dialog since fortune_cmd_label
+		 * might need to be hidden 
+		 * The dialog will be shown with gtk_window_present later */
+		gtk_widget_show (scrolled);
+		gtk_widget_show (fish->fortune_view);
+		gtk_widget_show (fish->fortune_label);
 	}
 
-	gtk_window_set_screen (GTK_WINDOW (fish->fortune_dialog),
-			       gtk_widget_get_screen (GTK_WIDGET (fish)));
-	gtk_window_present (GTK_WINDOW (fish->fortune_dialog));
+	if (!user_command) {
+		char *command;
+		char * text;
+
+		command = g_strdup_printf ("<tt>%s</tt>", argv[0]);
+		text = g_strdup_printf (_("The configured command is not "
+					  "working and has been replaced by: "
+					  "%s"), command);
+		gtk_label_set_markup (GTK_LABEL (fish->fortune_cmd_label),
+				      text);
+		g_free (command);
+		g_free (text);
+		gtk_widget_show (fish->fortune_cmd_label);
+	} else {
+		gtk_widget_hide (fish->fortune_cmd_label);
+	}
 
 	clear_fortune_text (fish);
 
-	g_spawn_command_line_sync (fortune_command, &output, NULL, NULL, &error);
+	gdk_spawn_on_screen_with_pipes (gtk_widget_get_screen (GTK_WIDGET (fish)),
+					NULL, argv, NULL,
+					G_SPAWN_SEARCH_PATH|G_SPAWN_STDERR_TO_DEV_NULL,
+					NULL, NULL, NULL, NULL, &output, NULL,
+					&error);
+
 	if (error) {
 		char *message;
 
 		message = g_strdup_printf (_("Unable to execute '%s'\n\nDetails: %s"),
-					   fortune_command, error->message);
+					   argv[0], error->message);
 		something_fishy_going_on (fish, message);
 		g_free (message);
 		g_error_free (error);
+		g_strfreev (argv);
+		return;
 	}
-	
-	g_free (fortune_command);
 
-	/* The output is not guarantied to be in UTF-8 format, most
-	 * likely it's just in ASCII-7 or in the user locale
-	  */
-	if (!g_utf8_validate (output, -1, NULL))
-		utf8_output = g_locale_to_utf8 (output, -1, NULL, NULL, NULL);
-	else
-		utf8_output = g_strdup (output);
+	fish->io_channel = g_io_channel_unix_new (output);
+	/* set the correct encoding if the locale is not using UTF-8 */
+	if (!g_get_charset (&charset))
+		g_io_channel_set_encoding(fish->io_channel, charset, &error);
+	if (error) {
+		char *message;
 
-	if (utf8_output)
-		insert_fortune_text (fish, utf8_output);
-	else
-		insert_fortune_text (fish, 
-				     _("You do not have fortune installed "
-				       "or you have not specified a program "
-				       "to run.\n\nPlease refer to fish "
-				       "properties dialog."));
+		message = g_strdup_printf (_("Unable to read from '%s'\n\nDetails: %s"),
+					   argv[0], error->message);
+		something_fishy_going_on (fish, message);
+		g_free (message);
+		g_error_free (error);
+		g_strfreev (argv);
+		return;
+	}
 
-	g_free (output);
-	g_free (utf8_output);
+	g_strfreev (argv);
+
+	fish->source_id = g_io_add_watch (fish->io_channel,
+					  G_IO_IN|G_IO_ERR|G_IO_HUP|G_IO_NVAL,
+					  fish_read_output, fish);
+
+	gtk_window_set_screen (GTK_WINDOW (fish->fortune_dialog),
+			       gtk_widget_get_screen (GTK_WIDGET (fish)));
+	gtk_window_present (GTK_WINDOW (fish->fortune_dialog));
 }
 
 static void
@@ -849,7 +989,7 @@ command_changed_notify (GConfClient *client,
 	
 	value = gconf_value_get_string (entry->value);
 
-	if (!value [0] || (fish->command && !strcmp (fish->command, value)))
+	if (fish->command && !strcmp (fish->command, value))
 		return;
 
 	if (fish->command)
@@ -1126,21 +1266,8 @@ load_fish_image (FishApplet *fish)
 
 	if (g_path_is_absolute (fish->image))
 		path = g_strdup (fish->image);
-
-	else {
-		path = gnome_program_locate_file (
-					NULL, GNOME_FILE_DOMAIN_PIXMAP,
-					fish->image, FALSE, NULL);
-		if (!path)
-			path = gnome_program_locate_file (
-					NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
-					fish->image, FALSE, NULL);
-	}
-
-	if (!path) {
-		g_warning ("Cannot locate '%s'", fish->image);
-		return FALSE;
-	}
+	else
+		path = g_strdup_printf ("%s/%s", FISH_ICONDIR, fish->image);
 
 	pixbuf = gdk_pixbuf_new_from_file (path, &error);
 	if (error) {
@@ -1366,6 +1493,41 @@ fish_applet_change_orient (PanelApplet       *applet,
 		update_pixmap (fish);
 }
 
+static void
+fish_applet_change_background (PanelApplet               *panel_applet,
+			       PanelAppletBackgroundType  type,
+			       GdkColor                  *color,
+			       GdkPixmap                 *pixmap)
+{
+	FishApplet *applet;
+	GtkRcStyle *rc_style;
+	GtkStyle   *style;
+
+	applet = (FishApplet *) panel_applet;
+
+	/* reset style */
+	gtk_widget_set_style (GTK_WIDGET (applet), NULL);
+	rc_style = gtk_rc_style_new ();
+	gtk_widget_modify_style (GTK_WIDGET (applet), rc_style);
+	g_object_unref (rc_style);
+
+	switch (type) {
+	case PANEL_NO_BACKGROUND:
+		break;
+	case PANEL_COLOR_BACKGROUND:
+		gtk_widget_modify_bg (GTK_WIDGET (applet),
+				      GTK_STATE_NORMAL, color);
+		break;
+	case PANEL_PIXMAP_BACKGROUND:
+		style = gtk_style_copy (GTK_WIDGET (applet)->style);
+		if (style->bg_pixmap[GTK_STATE_NORMAL])
+			g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
+		style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
+		gtk_widget_set_style (GTK_WIDGET (applet), style);
+		break;
+	}
+}
+
 static void
 change_water (FishApplet *fish)
 {
@@ -1512,6 +1674,22 @@ static const BonoboUIVerb fish_menu_verbs [] = {
         BONOBO_UI_VERB_END
 };
 
+static void
+fish_migrate_to_210 (FishApplet *fish)
+{
+	char *new_image;
+
+	g_assert (fish->image);
+
+	if (!strncmp (fish->image, "fish/", 5)) {
+		new_image = g_strdup (fish->image + 5);
+		g_free (fish->image);
+		fish->image = new_image;
+		panel_applet_gconf_set_string (PANEL_APPLET (fish), "image",
+					       fish->image, NULL);
+	}
+}
+
 static gboolean
 fish_applet_fill (FishApplet *fish)
 {
@@ -1541,7 +1719,8 @@ fish_applet_fill (FishApplet *fish)
 		error = NULL;
 	}
 	if (!fish->image)
-		fish->image = g_strdup ("fish/fishanim.png"); /* Fallback */
+		fish->image = g_strdup ("fishanim.png"); /* Fallback */
+	fish_migrate_to_210 (fish);
 
 	fish->command = panel_applet_gconf_get_string (applet, "command", &error);
 	if (error) {
@@ -1549,8 +1728,6 @@ fish_applet_fill (FishApplet *fish)
 		g_error_free (error);
 		error = NULL;
 	}
-	if (!fish->command)
-		fish->command = g_strdup ("fortune"); /* Fallback */
 
 	fish->n_frames = panel_applet_gconf_get_int (applet, "frames", &error);
 	if (error) {
@@ -1596,6 +1773,7 @@ fish_applet_fill (FishApplet *fish)
 					      NULL);
 	}
 
+	gtk_window_set_default_icon_name ("gnome-fish");
 	setup_fish_widget (fish);
 
 	return TRUE;
@@ -1667,6 +1845,12 @@ fish_applet_destroy (GtkObject *object)
 		gtk_widget_destroy (fish->fortune_dialog);
 	fish->fortune_dialog = NULL;
 
+	if (fish->source_id)
+		g_source_remove (fish->source_id);
+	fish->source_id = 0;
+
+	fish_close_channel (fish);
+
 	destroy_tooltip (fish);
 
 	GTK_OBJECT_CLASS (parent_class)->destroy (object);
@@ -1716,8 +1900,12 @@ fish_applet_instance_init (FishApplet      *fish,
 	fish->fortune_dialog = NULL;
 	fish->fortune_view   = NULL;
 	fish->fortune_label  = NULL;
+	fish->fortune_cmd_label = NULL;
 	fish->fortune_buffer = NULL;
 
+	fish->source_id  = 0;
+	fish->io_channel = NULL;
+
 	for (i = 0; i < N_FISH_PREFS; i++)
 		fish->listeners [i] = 0;
 
@@ -1736,6 +1924,7 @@ fish_applet_class_init (FishAppletClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	applet_class->change_orient = fish_applet_change_orient;
+	applet_class->change_background = fish_applet_change_background;
 
 	gtkobject_class->destroy = fish_applet_destroy;
 
