@@ -23,11 +23,10 @@
 
 #include <gtk/gtk.h>
 #include <libbonobo.h>
-#include <libgnome/libgnome.h>
-#include <libgnomeui/libgnomeui.h>
 #include <glade/glade-xml.h>
 #include <libwnck/libwnck.h>
 #include <gconf/gconf-client.h>
+#include <libgnomeui/gnome-help.h>
 
 #include "workspace-switcher.h"
 
@@ -147,6 +146,39 @@ applet_change_orient (PanelApplet       *applet,
 		gtk_label_set_text (GTK_LABEL (pager->label_row_col), pager->orientation == GTK_ORIENTATION_HORIZONTAL ? _("rows") : _("columns"));	
 }
 
+static void
+applet_change_background (PanelApplet               *applet,
+			  PanelAppletBackgroundType  type,
+			  GdkColor                  *color,
+			  GdkPixmap                 *pixmap,
+			  PagerData                 *pager)
+{
+	GtkRcStyle *rc_style;
+	GtkStyle   *style;
+
+	/* reset style */
+	gtk_widget_set_style (GTK_WIDGET (pager->applet), NULL);
+	rc_style = gtk_rc_style_new ();
+	gtk_widget_modify_style (GTK_WIDGET (pager->applet), rc_style);
+	g_object_unref (rc_style);
+
+	switch (type) {
+	case PANEL_NO_BACKGROUND:
+		break;
+	case PANEL_COLOR_BACKGROUND:
+		gtk_widget_modify_bg (GTK_WIDGET (pager->applet),
+				      GTK_STATE_NORMAL, color);
+		break;
+	case PANEL_PIXMAP_BACKGROUND:
+		style = gtk_style_copy (GTK_WIDGET (pager->applet)->style);
+		if (style->bg_pixmap[GTK_STATE_NORMAL])
+			g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
+		style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
+		gtk_widget_set_style (GTK_WIDGET (pager->applet), style);
+		break;
+	}
+}
+
 static void 
 response_cb (GtkWidget *widget,
 	     int        id,
@@ -428,6 +460,10 @@ workspace_switcher_applet_fill (PanelApplet *applet)
 			  "change_orient",
 			  G_CALLBACK (applet_change_orient),
 			  pager);
+	g_signal_connect (G_OBJECT (pager->applet),
+			  "change_background",
+			  G_CALLBACK (applet_change_background),
+			  pager);
 	
 	panel_applet_setup_menu_from_file (PANEL_APPLET (pager->applet),
 					   NULL,
@@ -490,18 +526,17 @@ display_about_dialog (BonoboUIComponent *uic,
 		      PagerData         *pager,
 		      const gchar       *verbname)
 {
-	GdkPixbuf *pixbuf = NULL;
-	gchar *file;
-	
 	static const gchar *authors[] =
 	{
 		"Alexander Larsson <alla@lysator.liu.se>",
 		NULL
 	};
 	const char *documenters [] = {
-	  NULL
+                "John Fleck <jfleck@inkstain.net>",
+                "Sun GNOME Documentation Team <gdocteam@sun.com>",
+                NULL
 	};
-	const char *translator_credits = _("translator_credits");
+	const char *translator_credits = _("translator-credits");
 
 	if (pager->about) {
 		gtk_window_set_screen (GTK_WINDOW (pager->about),
@@ -510,27 +545,23 @@ display_about_dialog (BonoboUIComponent *uic,
 		return;
 	}
 
-	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "gnome-workspace.png", TRUE, NULL);
-	pixbuf = gdk_pixbuf_new_from_file (file, NULL);
-	g_free(file);
+	pager->about = gtk_about_dialog_new ();
+	g_object_set (pager->about,
+		      "name",  _("Workspace Switcher"),
+		      "version", VERSION,
+		      "copyright", "Copyright \xc2\xa9 2001-2002 Red Hat, Inc.",
+		      "comments", _("The Workspace Switcher shows you a small version of your workspaces that lets you manage your windows."),
+		      "authors", authors,
+		      "documenters", documenters,
+		      "translator_credits", strcmp (translator_credits, "translator-credits") != 0 ? translator_credits : NULL,
+		      "logo_icon_name", "gnome-workspace",
+		      NULL);
 
-	pager->about = gnome_about_new (_("Workspace Switcher"), VERSION,
-				 "Copyright \xc2\xa9 2001-2002 Red Hat, Inc.",
-				 _("The Workspace Switcher shows you a small version of your workspaces that lets you manage your windows."),
-				 authors,
-				 documenters,
-				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
-				 pixbuf);
-	
 	gtk_window_set_wmclass (GTK_WINDOW (pager->about), "pager", "Pager");
 	gtk_window_set_screen (GTK_WINDOW (pager->about),
 			       gtk_widget_get_screen (pager->applet));
 
-	if (pixbuf) {
-		gtk_window_set_icon (GTK_WINDOW (pager->about), pixbuf);
-		g_object_unref (pixbuf);
-	}
-	
+	gtk_window_set_icon_name (GTK_WINDOW (pager->about), "gnome-workspace"); 
 	g_signal_connect (G_OBJECT(pager->about), "destroy",
 			  (GCallback)gtk_widget_destroyed, &pager->about);
 	
@@ -902,8 +933,8 @@ display_properties_dialog (BonoboUIComponent *uic,
 		g_object_unref (G_OBJECT (xml));
 	}
 
-	gnome_window_icon_set_from_file (GTK_WINDOW (pager->properties_dialog),
-	                                 GNOME_ICONDIR "/gnome-workspace.png");
+	gtk_window_set_icon_name (GTK_WINDOW (pager->properties_dialog),
+	                          "gnome-workspace");
 	gtk_window_set_screen (GTK_WINDOW (pager->properties_dialog),
 			       gtk_widget_get_screen (pager->applet));
 	gtk_window_present (GTK_WINDOW (pager->properties_dialog));
