@@ -27,7 +27,6 @@
 #include "launcher.h"
 
 #include "button-widget.h"
-#include "menu-fentry.h"
 #include "menu.h"
 #include "panel-util.h"
 #include "panel-config-global.h"
@@ -40,8 +39,6 @@
 #include "panel-multiscreen.h"
 #include "panel-lockdown.h"
 
-#include "quick-desktop-reader.h"
-
 #undef LAUNCHER_DEBUG
 
 static void properties_apply (Launcher *launcher);
@@ -134,7 +131,7 @@ panel_launcher_save_ditem (GnomeDesktopItem *ditem,
 	gnome_desktop_item_save (ditem, NULL, TRUE, &error);
 	if (error) {
 		panel_error_dialog (screen,
-				    "cannot_save_launcher",
+				    "cannot_save_launcher", TRUE,
 				    _("Cannot save launcher to disk"),
 				    "%s",
 				    error->message);
@@ -164,7 +161,7 @@ launch_url (Launcher *launcher)
 		GtkWidget *error_dialog;
 
 		error_dialog = panel_error_dialog (screen,
-						   "no_url_dialog",
+						   "no_url_dialog", TRUE,
 						   _("Cannot launch icon"),
 						   _("This launch icon does not specify a url to show."));
 		launcher_register_error_dialog (launcher, error_dialog);
@@ -177,6 +174,7 @@ launch_url (Launcher *launcher)
 	
 		error_dialog = panel_error_dialog (screen,
 						   "cannot_show_url_dialog",
+						   TRUE,
 						   _("Cannot show %s"),
 						   "%s",
 						   url,
@@ -186,9 +184,9 @@ launch_url (Launcher *launcher)
 	}
 }
 
-static void
-launch_cb (GtkWidget *widget,
-	   Launcher  *launcher)
+void
+launcher_launch (Launcher  *launcher,
+		 GtkWidget *widget)
 {
 	GnomeDesktopItem *item;
 
@@ -213,6 +211,7 @@ launch_cb (GtkWidget *widget,
 
 			error_dialog = panel_error_dialog (launcher_get_screen (launcher),
 							   "cannot_launch_icon",
+							   TRUE,
 							   _("Cannot launch icon"),
 							   "%s",
 							   error->message);
@@ -257,7 +256,7 @@ drag_data_received_cb (GtkWidget        *widget,
 	if (error) {
 		GtkWidget *error_dialog;
 		error_dialog = panel_error_dialog (launcher_get_screen (launcher),
-						   "cannot_launch_icon",
+						   "cannot_launch_icon", TRUE,
 						   _("Cannot launch icon"),
 						   "%s",
 						   error->message);
@@ -509,8 +508,8 @@ create_launcher (const char *location)
 			   G_CALLBACK (drag_drop_cb), launcher);
 	g_signal_connect (launcher->button, "drag_leave",
 			   G_CALLBACK (drag_leave_cb), launcher);
-	g_signal_connect (launcher->button, "clicked",
-			  G_CALLBACK (launch_cb), launcher);
+	g_signal_connect_swapped (launcher->button, "clicked",
+				  G_CALLBACK (launcher_launch), launcher);
 
 	launcher->destroy_handler =
 			g_signal_connect (launcher->button, "destroy",
@@ -817,6 +816,12 @@ load_launcher_applet (const char       *location,
 	if (!launcher->info)
 		return NULL;
 
+	panel_applet_add_callback (launcher->info,
+				   "launch",
+				   GTK_STOCK_EXECUTE,
+				   _("_Launch"),
+				   NULL);
+
 	panel_applet_add_callback (launcher->info,
 				   "properties",
 				   GTK_STOCK_PROPERTIES,
@@ -838,7 +843,6 @@ panel_launcher_ensure_hoarded (Launcher   *launcher,
 			       const char *id)
 {
 	GConfClient *client;
-	const char  *profile;
 	const char  *key;
 	const char  *new_location = NULL;
 
@@ -846,8 +850,7 @@ panel_launcher_ensure_hoarded (Launcher   *launcher,
 		return; /* already hoarded */
 
 	client  = panel_gconf_get_client ();
-	profile = panel_profile_get_name ();
-	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "launcher_location");
+	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "launcher_location");
 	if ( ! gconf_client_key_is_writable (client, key, NULL)) {
 		/* can't hoard */
 		launcher->non_writable = TRUE;
@@ -881,7 +884,6 @@ launcher_load_from_gconf (PanelWidget *panel_widget,
 {
 	GConfClient *client;
 	Launcher    *launcher;
-	const char  *profile;
 	const char  *key;
 	char        *launcher_location;
 
@@ -889,9 +891,8 @@ launcher_load_from_gconf (PanelWidget *panel_widget,
 	g_return_if_fail (id != NULL);
 
 	client  = panel_gconf_get_client ();
-	profile = panel_profile_get_name ();
 
-	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "launcher_location");
+	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "launcher_location");
 	launcher_location = gconf_client_get_string (client, key, NULL);
 
 	if (!launcher_location) {
@@ -949,6 +950,7 @@ really_add_launcher (GtkWidget *dialog, int response, gpointer data)
 		if (string_empty (gnome_desktop_item_get_localestring (ditem, GNOME_DESKTOP_ITEM_NAME))) {
 			err_dialog = panel_error_dialog (gtk_window_get_screen (GTK_WINDOW (dialog)),
 						         "cannot_create_launcher",
+							 TRUE,
 						         _("Cannot create launcher"),
 						         _("You have to specify a name."));
 			g_signal_connect_swapped (G_OBJECT (err_dialog),
@@ -966,6 +968,7 @@ really_add_launcher (GtkWidget *dialog, int response, gpointer data)
 		     string_empty (gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_URL)))) {
 			err_dialog = panel_error_dialog (gtk_window_get_screen (GTK_WINDOW (dialog)),
 							 "cannot_create_launcher",
+							 TRUE,
 							 _("Cannot create launcher"),
 							 _("You have to specify a valid URL or command."));
 			g_signal_connect_swapped (G_OBJECT (err_dialog),
@@ -1104,18 +1107,16 @@ panel_launcher_create_with_id (const char    *toplevel_id,
 			       const char    *location)
 {
 	GConfClient *client;
-	const char  *profile;
 	const char  *key;
 	char        *id;
 
 	g_return_if_fail (location != NULL);
 
 	client  = panel_gconf_get_client ();
-	profile = panel_profile_get_name ();
 
 	id = panel_profile_prepare_object_with_id (PANEL_OBJECT_LAUNCHER, toplevel_id, position, FALSE);
 
-	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "launcher_location");
+	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "launcher_location");
 	gconf_client_set_string (client, key, location, NULL);
 
 	panel_profile_add_to_list (PANEL_GCONF_OBJECTS, id);
