@@ -18,7 +18,6 @@
 #include <libbonoboui.h>
 #include <bonobo-activation/bonobo-activation.h>
 
-#include "menu-fentry.h"
 #include "menu.h"
 #include "panel.h"
 #include "panel-util.h"
@@ -43,10 +42,7 @@ GtkTooltips *panel_tooltips = NULL;
 
 GnomeIconTheme *panel_icon_theme = NULL;
 
-static char *profile_arg;
-
 static const struct poptOption options[] = {
-  {"profile", '\0', POPT_ARG_STRING, &profile_arg, 0, N_("Specify a profile name to load"), NULL},
   {NULL, '\0', 0, NULL, 0}
 };
 
@@ -78,8 +74,6 @@ main (int argc, char **argv)
 	panel_multiscreen_init ();
 	panel_init_stock_icons_and_items ();
 
-        init_menus ();
-
 	gconf_client_add_dir (panel_gconf_get_client (),
 			      "/desktop/gnome/interface",
 			      GCONF_CLIENT_PRELOAD_NONE,
@@ -87,7 +81,7 @@ main (int argc, char **argv)
 
 	panel_global_config_load ();
 	panel_lockdown_init ();
-	panel_profile_load (profile_arg);
+	panel_profile_load ();
 
 	panel_session_init (argv [0]);
 
