@@ -77,7 +77,7 @@ static GConfEnumStringPair panel_action_type_map [] = {
 
 /* Lock Screen
  */
-void
+static void
 panel_action_lock_screen (GtkWidget *widget)
 {
 	panel_lock_screen (gtk_widget_get_screen (widget));
@@ -159,7 +159,7 @@ panel_action_lock_invoke_menu (PanelActionButton *button,
 
 /* Log Out
  */
-void
+static void
 panel_action_logout (GtkWidget *widget)
 {
 	static int recursion_guard = 0;
@@ -176,7 +176,7 @@ panel_action_logout (GtkWidget *widget)
 
 /* Run Application
  */
-void
+static void
 panel_action_run_program (GtkWidget *widget)
 {
 	panel_run_dialog_present (gtk_widget_get_screen (widget), gtk_get_current_event_time ());
@@ -184,20 +184,41 @@ panel_action_run_program (GtkWidget *widget)
 
 /* Search For Files
  */
-void
+static void
 panel_action_search (GtkWidget *widget)
 {
+	GnomeDesktopItem *ditem;
 	GdkScreen *screen;
 	GError    *error = NULL;
 	char      *argv[2] = {"gnome-search-tool", NULL};
+	char      *file;
 
 	screen = gtk_widget_get_screen (widget);
+	file = panel_lookup_in_data_dirs ("applications/gnome-search-tool.desktop");
 
-	if (!gdk_spawn_on_screen (screen, NULL, argv, NULL,
+	if (file) {
+		ditem = gnome_desktop_item_new_from_file (file, 0, &error);
+		if (ditem) {
+			g_assert (error == NULL);	
+			panel_ditem_launch (ditem, NULL, 0, screen, &error);
+			gnome_desktop_item_unref (ditem);
+		}
+		if (error) {
+			panel_error_dialog (screen,
+			                    "cannot_exec_gnome-search-tool", TRUE,
+			                    _("Cannot execute '%s'"),
+			                    "%s",
+			                    "gnome-search-tool",
+			                    error->message);
+			g_error_free (error);
+		}
+		g_free (file);
+	}
+	else if (!gdk_spawn_on_screen (screen, NULL, argv, NULL,
 				  G_SPAWN_SEARCH_PATH,
 				  NULL, NULL, NULL, &error)) {
 		panel_error_dialog (screen,
-				    "cannot_exec_gnome-search-tool",
+				    "cannot_exec_gnome-search-tool", TRUE,
 				    _("Cannot execute '%s'"),
 				    "%s",
 				    "gnome-search-tool",
@@ -208,7 +229,7 @@ panel_action_search (GtkWidget *widget)
 
 /* Take Screenshot
  */
-void
+static void
 panel_action_screenshot (GtkWidget *widget)
 {
 	GdkScreen *screen;
@@ -221,7 +242,7 @@ panel_action_screenshot (GtkWidget *widget)
 				  G_SPAWN_SEARCH_PATH,
 				  NULL, NULL, NULL, &error)) {
 		panel_error_dialog (screen,
-				    "cannot_exec_gnome-screenshot",
+				    "cannot_exec_gnome-screenshot", TRUE,
 				    _("Cannot execute '%s'"),
 				    "%s",
 				    "gnome-screenshot",
@@ -241,6 +262,7 @@ panel_action_force_quit (GtkWidget *widget)
 typedef struct {
 	PanelActionButtonType   type;
 	char                   *stock_icon;
+	char                   *icon_name;
 	char                   *text;
 	char                   *tooltip;
 	char                   *help_index;
@@ -262,7 +284,8 @@ static PanelAction actions [] = {
 	},
 	{
 		PANEL_ACTION_LOCK,
-		PANEL_STOCK_LOCKSCREEN,
+		NULL,
+		"gnome-lockscreen",
 		N_("Lock screen"),
 		N_("Protect your computer from unauthorized use"),
 		"gospanel-21",
@@ -274,7 +297,8 @@ static PanelAction actions [] = {
 	},
 	{
 		PANEL_ACTION_LOGOUT,
-		PANEL_STOCK_LOGOUT,
+		NULL,
+		"gnome-logout",
 		N_("Log Out"),
 		N_("Log out of this session to log in as a different user or to shut down the computer"),
 		"gospanel-20",
@@ -285,6 +309,7 @@ static PanelAction actions [] = {
 	{
 		PANEL_ACTION_RUN,
 		PANEL_STOCK_RUN,
+		NULL,
 		N_("Run Application..."),
 		N_("Run an Application by entering a command"),
 		"gospanel-555",
@@ -294,7 +319,8 @@ static PanelAction actions [] = {
 	},
 	{
 		PANEL_ACTION_SEARCH,
-		PANEL_STOCK_SEARCHTOOL,
+		NULL,
+		"gnome-searchtool",
 		N_("Search for Files..."),
 		N_("Find files, folders, and documents on your computer"),
 		"gospanel-554",
@@ -303,7 +329,8 @@ static PanelAction actions [] = {
 	},
 	{
 		PANEL_ACTION_SCREENSHOT,
-		PANEL_STOCK_SCREENSHOT,
+		NULL,
+		"applets-screenshooter",
 		N_("Take Screenshot..."),
 		N_("Take a screenshot of your desktop"),
 		"gospanel-553",
@@ -313,6 +340,7 @@ static PanelAction actions [] = {
 	{
 		PANEL_ACTION_FORCE_QUIT,
 		PANEL_STOCK_FORCE_QUIT,
+		NULL,
 		N_("Force Quit"),
 		N_("Force a misbehaving application to quit"),
 		"gospanel-563",
@@ -333,6 +361,16 @@ panel_action_get_is_disabled (PanelActionButtonType type)
 	return FALSE;
 }
 
+GCallback
+panel_action_get_invoke (PanelActionButtonType type)
+{
+	g_return_val_if_fail (type > PANEL_ACTION_NONE && type < PANEL_ACTION_LAST, NULL);
+
+	g_assert (actions[type].invoke != NULL);
+
+	return G_CALLBACK (actions[type].invoke);
+}
+
 G_CONST_RETURN char*
 panel_action_get_stock_icon (PanelActionButtonType type)
 {
@@ -341,6 +379,14 @@ panel_action_get_stock_icon (PanelActionButtonType type)
 	return actions[type].stock_icon;
 }
 
+G_CONST_RETURN char*
+panel_action_get_icon_name (PanelActionButtonType type)
+{
+	g_return_val_if_fail (type > PANEL_ACTION_NONE && type < PANEL_ACTION_LAST, NULL);
+
+	return actions[type].icon_name;
+}
+
 G_CONST_RETURN char*
 panel_action_get_text (PanelActionButtonType type)
 {
@@ -582,7 +628,10 @@ panel_action_button_set_type (PanelActionButton     *button,
 
 	button->priv->type = type;
 
-	button_widget_set_stock_id (BUTTON_WIDGET (button), actions [type].stock_icon);
+	if (actions [type].icon_name != NULL)
+		button_widget_set_icon_name (BUTTON_WIDGET (button), actions [type].icon_name);
+	else
+		button_widget_set_stock_id (BUTTON_WIDGET (button), actions [type].stock_icon);
 
 	gtk_tooltips_set_tip (panel_tooltips, GTK_WIDGET (button),
 			      _(actions [type].tooltip), NULL);
@@ -633,7 +682,10 @@ panel_action_button_connect_to_gconf (PanelActionButton *button)
 static void
 panel_action_button_style_set (PanelActionButton *button)
 {
-	button_widget_set_stock_id (BUTTON_WIDGET (button), actions [button->priv->type].stock_icon);
+	if (actions [button->priv->type].icon_name != NULL)
+		button_widget_set_icon_name (BUTTON_WIDGET (button), actions [button->priv->type].icon_name);
+	else
+		button_widget_set_stock_id (BUTTON_WIDGET (button), actions [button->priv->type].stock_icon);
 }
 
 static void
@@ -849,8 +901,14 @@ panel_action_button_set_dnd_enabled (PanelActionButton *button,
 		gtk_drag_source_set (GTK_WIDGET (button), GDK_BUTTON1_MASK,
 				     dnd_targets, 1,
 				     GDK_ACTION_COPY | GDK_ACTION_MOVE);
-		gtk_drag_source_set_icon_stock (GTK_WIDGET (button),
-						actions [button->priv->type].stock_icon);
+		if (actions [button->priv->type].stock_icon != NULL)
+			gtk_drag_source_set_icon_stock (GTK_WIDGET (button),
+							actions [button->priv->type].stock_icon);
+		/* FIXME: waiting for bug #116577
+		else
+		gtk_drag_source_set_icon_name (GTK_WIDGET (button),
+						actions [button->priv->type].icon_name);
+						*/
 		GTK_WIDGET_SET_FLAGS (button, GTK_NO_WINDOW);
 	} else
 		gtk_drag_source_unset (GTK_WIDGET (button));
