@@ -25,14 +25,11 @@
 #include <config.h>
 #include <string.h>
 
-#include <libgnome/gnome-i18n.h>
 #include <libbonobo.h>
 
 #include "menu.h"
 
 #include "launcher.h"
-#include "menu-fentry.h"
-#include "menu-ditem.h"
 #include "panel.h"
 #include "drawer.h"
 #include "panel-applet-frame.h"
@@ -58,6 +55,8 @@ typedef struct {
 	GtkTreeModel *applet_model;
 	GtkTreeModel *application_model;
 
+	MenuTree     *menu_tree;
+
 	GSList       *applet_list;
 	GSList       *application_list;
 
@@ -81,14 +80,14 @@ typedef enum {
 
 typedef struct {
 	PanelAddtoItemType     type;
-	const char            *name;
-	const char            *description;
-	const char            *icon;
+	char                  *name;
+	char                  *description;
+	char                  *icon;
 	const char            *stock_icon;
 	PanelActionButtonType  action_type;
-	const char            *launcher_name;
-	const char            *menu_path;
-	const char            *iid;
+	char                  *launcher_path;
+	char                  *menu_path;
+	char                  *iid;
 	gboolean               static_data;
 } PanelAddtoItemInfo;
 
@@ -102,8 +101,8 @@ static PanelAddtoItemInfo special_addto_items [] = {
 	{ PANEL_ADDTO_LAUNCHER_NEW,
 	  N_("Custom Application Launcher"),
 	  N_("Create a new launcher"),
+	  "launcher-program",
 	  NULL,
-	  PANEL_STOCK_LAUNCHER,
 	  PANEL_ACTION_NONE,
 	  NULL,
 	  NULL,
@@ -113,8 +112,8 @@ static PanelAddtoItemInfo special_addto_items [] = {
 	{ PANEL_ADDTO_LAUNCHER_MENU,
 	  N_("Application Launcher..."),
 	  N_("Launch a program that is already in the GNOME menu"),
+	  "launcher-program",
 	  NULL,
-	  PANEL_STOCK_LAUNCHER,
 	  PANEL_ACTION_NONE,
 	  NULL,
 	  NULL,
@@ -150,8 +149,8 @@ static PanelAddtoItemInfo internal_addto_items [] = {
 	{ PANEL_ADDTO_DRAWER,
 	  N_("Drawer"),
 	  N_("A pop out drawer to store other items in"),
+	  "drawer",
 	  NULL,
-	  PANEL_STOCK_DRAWER,
 	  PANEL_ACTION_NONE,
 	  NULL,
 	  NULL,
@@ -220,9 +219,10 @@ panel_addto_append_internal_applets (GSList *list)
 		info->action_type = i;
 		info->name        = (char *) panel_action_get_text (i);
 		info->description = (char *) panel_action_get_tooltip (i);
+		info->icon        = (char *) panel_action_get_icon_name (i);
 		info->stock_icon  = (char *) panel_action_get_stock_icon (i);
 		info->iid         = (char *) panel_action_get_drag_id (i);
-		info->static_data = FALSE;
+		info->static_data = TRUE;
 
                 list = g_slist_append (list, info);
 	}
@@ -365,10 +365,10 @@ static GSList *
 panel_addto_query_applets (GSList *list)
 {
 	Bonobo_ServerInfoList *applet_list;
-	CORBA_Environment  env;
-	const GList *langs_glist;
-	GSList *langs_gslist;
-	int i;
+	CORBA_Environment   env;
+	const char * const *langs;
+	GSList             *langs_gslist;
+	int                 i;
 
 	CORBA_exception_init (&env);
 
@@ -380,20 +380,20 @@ panel_addto_query_applets (GSList *list)
 			   BONOBO_EX_REPOID (&env));
 
 		CORBA_exception_free (&env);
+		CORBA_free (applet_list);
 
 		return NULL;
 	}
 
 	CORBA_exception_free (&env);
 
-	/* Evil evil evil evil, we need to convery from a
-	   GList to a GSList */
+	langs = g_get_language_names ();
+
 	langs_gslist = NULL;
-	for (langs_glist = gnome_i18n_get_language_list ("LC_MESSAGES");
-	     langs_glist != NULL; langs_glist = langs_glist->next) {
-		langs_gslist = g_slist_append (langs_gslist,
-					       langs_glist->data);
-	}
+	for (i = 0; langs[i]; i++)
+		langs_gslist = g_slist_prepend (langs_gslist, (char *) langs[i]);
+
+	langs_gslist = g_slist_reverse (langs_gslist);
 
 	for (i = 0; i < applet_list->_length; i++) {
 		Bonobo_ServerInfo *info;
@@ -418,16 +418,17 @@ panel_addto_query_applets (GSList *list)
 
 		applet = g_new0 (PanelAddtoItemInfo, 1);
 		applet->type = PANEL_ADDTO_APPLET;
-		applet->name = name;
-		applet->description = description;
-		applet->icon = icon;
-		applet->iid = info->iid;
+		applet->name = g_strdup (name);
+		applet->description = g_strdup (description);
+		applet->icon = g_strdup (icon);
+		applet->iid = g_strdup (info->iid);
 		applet->static_data = FALSE;
 
 		list = g_slist_append (list, applet);
 	}
 
 	g_slist_free (langs_gslist);
+	CORBA_free (applet_list);
 
 	return list;
 }
@@ -527,57 +528,68 @@ panel_addto_make_applet_model (PanelAddtoDialog *dialog)
 }
 
 static void
-panel_addto_make_application_list_r (GSList **parent,
-				     DirRec *dr)
+panel_addto_make_application_list (GSList            **parent_list,
+				   MenuTreeDirectory  *directory)
 {
-	PanelAddtoAppList *data;
-	FileRec *tfr;
-	GSList *li;
+	GSList *subdirs;
+	GSList *entries;
+	GSList *l;
 
-	if (dr == NULL)
-		return;
+	subdirs = menu_tree_directory_get_subdirs (directory);
+	for (l = subdirs; l; l = l->next) {
+		MenuTreeDirectory *subdir = l->data;
+		PanelAddtoAppList *data;
 
-	for (li = dr->recs; li != NULL; li = li->next) {
-		tfr = li->data;
+		data = g_new0 (PanelAddtoAppList, 1);
 
-		switch (tfr->type) {
-		case FILE_REC_DIR:
-			data = g_new0 (PanelAddtoAppList, 1);
-			data->item_info.type = PANEL_ADDTO_MENU;
-			data->item_info.name = tfr->fullname;
-			data->item_info.description = tfr->comment;
-			data->item_info.icon = tfr->icon;
-			data->item_info.menu_path = tfr->name;
-			/* We should set the iid here to something and do
-			 * iid = g_strdup_printf ("MENU:%s", tfr->name)
-			 * but this means we'd have to free the iid later
-			 * and this would complexify too much the free
-			 * function.
-			 * So the iid is built when we select the row. */
-			tfr = fr_check_and_reread (tfr);
-			panel_addto_make_application_list_r (&(data->children),
-							     (DirRec *) tfr);
-			*parent = g_slist_append (*parent, data);
-			break;
-		case FILE_REC_FILE:
-			data = g_new0 (PanelAddtoAppList, 1);
-			data->item_info.type = PANEL_ADDTO_LAUNCHER;
-			data->item_info.name = tfr->fullname;
-			data->item_info.description = tfr->comment;
-			data->item_info.icon = tfr->icon;
-			data->item_info.launcher_name = tfr->name;
-			*parent = g_slist_append (*parent, data);
-			break;
-		default:
-			break;
-		}
+		data->item_info.type        = PANEL_ADDTO_MENU;
+		data->item_info.name        = g_strdup (menu_tree_directory_get_name (subdir));
+		data->item_info.description = g_strdup (menu_tree_directory_get_comment (subdir));
+		data->item_info.icon        = g_strdup (menu_tree_directory_get_icon (subdir));
+		data->item_info.menu_path   = menu_tree_directory_make_path (subdir, NULL);
+		data->item_info.static_data = FALSE;
+
+		/* We should set the iid here to something and do
+		 * iid = g_strdup_printf ("MENU:%s", tfr->name)
+		 * but this means we'd have to free the iid later
+		 * and this would complexify too much the free
+		 * function.
+		 * So the iid is built when we select the row.
+		 */
+
+		*parent_list = g_slist_append (*parent_list, data);
+
+		panel_addto_make_application_list (&data->children, subdir);
+
+		menu_tree_directory_unref (subdir);
 	}
+	g_slist_free (subdirs);
+
+	entries = menu_tree_directory_get_entries (directory);
+	for (l = entries; l; l = l->next) {
+		MenuTreeEntry     *entry = l->data;
+		PanelAddtoAppList *data;
+
+		data = g_new0 (PanelAddtoAppList, 1);
+
+		data->item_info.type          = PANEL_ADDTO_LAUNCHER;
+		data->item_info.name          = g_strdup (menu_tree_entry_get_name (entry));
+		data->item_info.description   = g_strdup (menu_tree_entry_get_comment (entry));
+		data->item_info.icon          = g_strdup (menu_tree_entry_get_icon (entry));
+		data->item_info.launcher_path = g_strdup (menu_tree_entry_get_desktop_file_path (entry));
+		data->item_info.static_data = FALSE;
+
+		*parent_list = g_slist_append (*parent_list, data);
+
+		menu_tree_entry_unref (entry);
+	}
+	g_slist_free (entries);
 }
 
 static void
-panel_addto_make_application_model_r (GtkTreeStore *store,
-				      GtkTreeIter  *parent,
-				      GSList       *app_list)
+panel_addto_populate_application_model (GtkTreeStore *store,
+					GtkTreeIter  *parent,
+					GSList       *app_list)
 {
 	PanelAddtoAppList *data;
 	GtkTreeIter        iter;
@@ -606,16 +618,18 @@ panel_addto_make_application_model_r (GtkTreeStore *store,
 		g_free (text);
 
 		if (data->children != NULL) 
-			panel_addto_make_application_model_r (store, &iter,
-							      data->children);
+			panel_addto_populate_application_model (store,
+								&iter,
+								data->children);
 	}
 }
 
 static GtkTreeModel *
 panel_addto_make_application_model (PanelAddtoDialog *dialog)
 {
-	GtkTreeStore *store;
-	DirRec *root;
+	GtkTreeStore      *store;
+	MenuTree          *tree;
+	MenuTreeDirectory *root;
 
 	store = gtk_tree_store_new (NUMBER_COLUMNS,
 				    GDK_TYPE_PIXBUF,
@@ -623,11 +637,16 @@ panel_addto_make_application_model (PanelAddtoDialog *dialog)
 				    G_TYPE_POINTER,
 				    G_TYPE_STRING);
 
-	root = (DirRec *) fr_get_dir ("applications:/");
-	panel_addto_make_application_list_r (&(dialog->application_list),
-					     root);
-	panel_addto_make_application_model_r (store, NULL,
-					      dialog->application_list);
+	tree = menu_tree_lookup ("applications.menu");
+
+	if ((root = menu_tree_get_root_directory (tree))) {
+		panel_addto_make_application_list (&dialog->application_list, root);
+		panel_addto_populate_application_model (store, NULL, dialog->application_list);
+
+		menu_tree_directory_unref (root);
+	}
+
+	menu_tree_unref (tree);
 
 	return GTK_TREE_MODEL (store);
 }
@@ -660,7 +679,7 @@ panel_addto_add_item (PanelAddtoDialog   *dialog,
 	case PANEL_ADDTO_LAUNCHER:
 		panel_launcher_create (dialog->panel_widget->toplevel,
 				       dialog->insertion_position,
-				       item_info->launcher_name);
+				       item_info->launcher_path);
 		break;
 	case PANEL_ADDTO_LAUNCHER_NEW:
 		ask_about_launcher (NULL, dialog->panel_widget,
@@ -768,6 +787,37 @@ panel_addto_present_applets (PanelAddtoDialog *dialog)
 	gtk_widget_set_sensitive (dialog->back_button, FALSE);
 }
 
+static void
+panel_addto_dialog_free_item_info (PanelAddtoItemInfo *item_info)
+{
+	if (item_info == NULL || item_info->static_data)
+		return;
+
+	if (item_info->name != NULL)
+		g_free (item_info->name);
+	item_info->name = NULL;
+
+	if (item_info->description != NULL)
+		g_free (item_info->description);
+	item_info->description = NULL;
+
+	if (item_info->icon != NULL)
+		g_free (item_info->icon);
+	item_info->icon = NULL;
+
+	if (item_info->iid != NULL)
+		g_free (item_info->iid);
+	item_info->iid = NULL;
+
+	if (item_info->launcher_path != NULL)
+		g_free (item_info->launcher_path);
+	item_info->launcher_path = NULL;
+
+	if (item_info->menu_path != NULL)
+		g_free (item_info->menu_path);
+	item_info->menu_path = NULL;
+}
+
 static void
 panel_addto_dialog_free_application_list (GSList *application_list)
 {
@@ -782,6 +832,7 @@ panel_addto_dialog_free_application_list (GSList *application_list)
 		if (data->children) {
 			panel_addto_dialog_free_application_list (data->children);
 		}
+		panel_addto_dialog_free_item_info (&data->item_info);
 		g_free (data);
 	}
 	g_slist_free (application_list);
@@ -808,6 +859,7 @@ panel_addto_dialog_free (PanelAddtoDialog *dialog)
 
 		applet = (PanelAddtoItemInfo *) item->data;
 		if (!applet->static_data) {
+			panel_addto_dialog_free_item_info (applet);
 			g_free (applet);
 		}
 	}
@@ -815,6 +867,10 @@ panel_addto_dialog_free (PanelAddtoDialog *dialog)
 
 	panel_addto_dialog_free_application_list (dialog->application_list);
 
+	if (dialog->menu_tree)
+		menu_tree_unref (dialog->menu_tree);
+	dialog->menu_tree = NULL;
+
 	g_free (dialog);
 }
 
@@ -900,7 +956,6 @@ panel_addto_selection_changed (GtkTreeSelection *selection,
 		gtk_button_set_use_stock (GTK_BUTTON (dialog->add_button),
 					  TRUE);
 	} else {
-		//FIXME
 		gtk_button_set_label (GTK_BUTTON (dialog->add_button),
 				      GTK_STOCK_ADD);
 		gtk_button_set_use_stock (GTK_BUTTON (dialog->add_button),
@@ -912,7 +967,7 @@ panel_addto_selection_changed (GtkTreeSelection *selection,
 		switch (data->type) {
 		case PANEL_ADDTO_LAUNCHER:
 			panel_addto_setup_launcher_drag (GTK_TREE_VIEW (dialog->tree_view),
-							 data->launcher_name);
+							 data->launcher_path);
 			break;
 		case PANEL_ADDTO_APPLET:
 			panel_addto_setup_applet_drag (GTK_TREE_VIEW (dialog->tree_view),
